

puts "loading sys.V785.tcl"

variable registerlist

lappend registerlist [list NetVar.V785_NumSamples  "-n V785_NumSamples -t int"]
lappend registerlist [list NetVar.V785_VMERetries  "-n V785_VMERetries -t int"]
lappend registerlist [list NetVar.V785_BERREnable  "-n V785_BERREnable -t int"]
lappend registerlist [list NetVar.V785_VMEBufSize  "-n V785_VMEBufSize -t int"]


proc Set_V785_Options {} {puts "Set_V785_Options"
    variable Options

    load_Options

   if {![info exists Options(V785_MaxEvtsPerBuf)]}         {set Options(V785_MaxEvtsPerBuf) 1}
   if {![info exists Options(V785_MaxBLT)]}                {set Options(V785_MaxBLT) 1}
   if {![info exists Options(V785_MemBufSize)]}            {set Options(V785_MemBufSize) 1024}
   if {![info exists Options(V785_VMEBufSize)]}            {set Options(V785_VMEBufSize) 16384}

   if {![info exists Options(V785_HistEnable)]} {set Options(V785_HistEnable) 1}
   if {![info exists Options(V785_channels)]}   {set Options(V785_channels) 4096}
   if {![info exists Options(V785_offset)]}     {set Options(V785_offset) 0}
   if {![info exists Options(V785_shift)]}      {set Options(V785_shift) 0}

   if {![info exists Options(V785_GroupBase)]}  {set Options(V785_GroupBase) 160}   ;#  default Group Base

    update_Options
}

proc V785_Configure {} {puts "V785_Configure"
    variable Options

    set Options(V785_VMEBufSize) [ReadNetVar NetVar.V785_VMEBufSize]

    update_Options
}

proc Dump_Raw_VME_Data_V785 {data} {
     
     set info ""

     foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 24 & 7]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 16 & 0x3f]
             set un [expr $word >> 13 & 1]
             set ov [expr $word >> 12 & 1]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i; un= %3i; ov= %3i; data= 0x%5x" $word $event_type $channel $un $ov $adc]"
            }
            2 {
             set event_type "header"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word >> 8 & 0x3f]
             append info "[format "0x%8x    %8s   module= %5i; channels= %6i" $word $event_type $geo $counter]"
            }
            4 {
             set event_type "trailer"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word & 0xffffff]
             append info "[format "0x%8x    %8s   module= %5i; event data counter= %10i" $word $event_type $geo $counter]"
            }
            default {
             append info "$word - not valid datum\n"
            }
         }
         append info "\n"
     }
   return $info
}


proc V785_DAQ-go {} {puts "V785_DAQ-go"

      WriteVMERegisters v785#*.Bit_Set_2 4     ;# clear the data
      WriteVMERegisters v785#*.Bit_Clear_2 4

      WriteVMERegisters v785#*.Bit_Set_2 0x1000   ;# always generate event header
      WriteVMERegisters v785#*.Ctrl1 0x24         ;# enable BLKEND and BERR
}

proc V785_DAQ-halt {} {puts "V785_DAQ-halt"}

proc V785_DAQ-setup {} {puts "V785_DAQ-setup"

      V785_Configure

      WriteVMERegisters v785#*.Bit_Set_1 0x80     ;# software reset the board
      WriteVMERegisters v785#*.Bit_Clear_1 0x80

      WriteVMERegisters v785#*.Bit_Set_2 0x1000   ;#  always generate event header
      WriteVMERegisters v785#*.Bit_Set_2 0x8      ;# no overflow suppression 
}

proc V785_DAQ-flush {} {puts "V785_DAQ-flush"

      WriteVMERegisters v785#*.Bit_Set_2 4     ;# clear the data
      WriteVMERegisters v785#*.Bit_Clear_2 4
}

proc V785_DAQ-reset {} {puts "V785_DAQ-reset"}

proc define_V785_config {} {puts "define_V785_config"
   variable Options

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v785#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "785 0 [format "0x%x" $Options(V785_GroupBase)]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr v785#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register v785#$i.Output_Buffer"
          error $m
      }

      DefineConfig "785 $i [string range [first $m] 2 end]"
   }
}


proc create_V785_spectra {} {puts "create_V785_spectra"
    variable Options 

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v785#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }
   set maxadc 32

   for {set i 1} {$i <= $maxmodule} {incr i} {

      for {set j 0} {$j < $maxadc} {incr j} {

         if {$Options(V785_HistEnable) > 0} {

#    histograms for the ADC

           set spectrumname v785#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V785_channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr $j + (($i-1) * $maxadc) + $Options(V785_GroupBase)] 
              map_spectrum  "$spectrumname $group 0 $Options(V785_offset) $Options(V785_shift)"
           }
         }
      }
   }
}



